import * as React from 'react';
import { StyleSheet } from 'react-native';
import { RNIUtilitiesModule } from 'react-native-ios-utilities';
import { RNIAdaptiveModalViewModule } from './RNIAdaptiveModalViewModule';
import { RNIAdaptiveModalNativeView } from './RNIAdaptiveModalNativeView';
export class RNIAdaptiveModalView extends React.PureComponent {
    nativeRef;
    reactTag;
    constructor(props) {
        super(props);
    }
    ;
    componentWillUnmount() {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        RNIUtilitiesModule.notifyOnComponentWillUnmount(reactTag, {
            shouldForceCleanup: true,
            shouldIgnoreCleanupTriggers: false,
        });
    }
    ;
    getNativeRef = () => {
        return this.nativeRef;
    };
    getNativeReactTag = () => {
        // @ts-ignore
        return this.nativeRef?.nativeTag ?? this.reactTag;
    };
    notifyDidLayoutSubviews = async () => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.notifyDidLayoutSubviews(reactTag);
    };
    clearSnapPointOverride = async () => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.clearSnapPointOverride(reactTag);
    };
    presentModal = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.presentModal(reactTag, commandConfig);
    };
    dismissModal = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.dismissModal(reactTag, commandConfig);
    };
    snapTo = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapTo(reactTag, commandConfig);
    };
    snapToOverride = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapToOverride(reactTag, commandConfig);
    };
    snapToClosestSnapPoint = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapToClosestSnapPoint(reactTag, commandConfig ?? {});
    };
    snapToPrevSnapPointIndex = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapToPrevSnapPointIndex(reactTag, commandConfig ?? {});
    };
    snapToCurrentSnapPointIndex = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapToCurrentSnapPointIndex(reactTag, commandConfig ?? {});
    };
    snapToNextSnapPointIndex = async (commandConfig) => {
        const reactTag = this.getNativeReactTag();
        if (typeof reactTag !== 'number')
            return;
        await RNIAdaptiveModalViewModule.snapToNextSnapPointIndex(reactTag, commandConfig ?? {});
    };
    // Event Handlers
    // --------------
    _handleOnLayout = ({ nativeEvent }) => {
        // @ts-ignore
        this.reactTag = nativeEvent.target;
    };
    _handleOnNativeRef = (ref) => {
        this.nativeRef = ref;
    };
    render() {
        return React.createElement(RNIAdaptiveModalNativeView, {
            ...this.props,
            ...((this.reactTag == null) && {
                onLayout: this._handleOnLayout,
            }),
            // @ts-ignore
            ref: this._handleOnNativeRef,
            style: [
                this.props.style,
                styles.nativeView
            ],
        });
    }
    ;
}
;
const styles = StyleSheet.create({
    nativeView: {},
});
//# sourceMappingURL=RNIAdaptiveModalView.js.map